@@ -1,5 +1,5 @@
 /* stty -- change and print terminal line settings
-   Copyright (C) 1990-1999 Free Software Foundation, Inc.
+   Copyright (C) 1990-2000 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -64,6 +64,7 @@
 #include "long-options.h"
 #include "error.h"
 #include "xstrtol.h"
+#include "closeout.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "stty"
@@ -134,6 +135,9 @@
 #if defined(VREPRINT) && !defined(CRPRNT)
 # define CRPRNT Control ('r')
 #endif
+#if defined(CREPRINT) && !defined(CRPRNT)
+# define CRPRNT Control ('r')
+#endif
 #if defined(VWERASE) && !defined(CWERASE)
 # define CWERASE Control ('w')
 #endif
@@ -379,6 +383,10 @@ static struct control_info control_info[] =
 #endif
 #ifdef VREPRINT
   {"rprnt", CRPRNT, VREPRINT},
+#else
+# ifdef CREPRINT /* HPUX 10.20 needs this */
+  {"rprnt", CRPRNT, CREPRINT},
+# endif
 #endif
 #ifdef VWERASE
   {"werase", CWERASE, VWERASE},
@@ -467,12 +475,13 @@ wrapf (const char *message,...)
   vsprintf (buf, message, args);
   va_end (args);
   buflen = strlen (buf);
-  if (current_col + (current_col > 0) + buflen >= max_col)
+  if (0 < current_col
+      && max_col <= current_col + (0 < current_col) + buflen)
     {
       putchar ('\n');
       current_col = 0;
     }
-  if (current_col > 0)
+  if (0 < current_col)
     {
       putchar (' ');
       current_col++;
@@ -490,19 +499,19 @@ usage (int status)
   else
     {
       printf (_("\
-Usage: %s [-F device] [--file=device] [SETTING]...\n\
-  or:  %s [-F device] [--file=device] [-a|--all]\n\
-  or:  %s [-F device] [--file=device] [-g|--save]\n\
+Usage: %s [-F DEVICE] [--file=DEVICE] [SETTING]...\n\
+  or:  %s [-F DEVICE] [--file=DEVICE] [-a|--all]\n\
+  or:  %s [-F DEVICE] [--file=DEVICE] [-g|--save]\n\
 "),
 	      program_name, program_name, program_name);
       printf (_("\
 Print or change terminal characteristics.\n\
 \n\
-  -a, --all       print all current settings in human-readable form\n\
-  -g, --save      print all current settings in a stty-readable form\n\
-  -F, --file      open and use the specified device instead of stdin\n\
-      --help      display this help and exit\n\
-      --version   output version information and exit\n\
+  -a, --all          print all current settings in human-readable form\n\
+  -g, --save         print all current settings in a stty-readable form\n\
+  -F, --file=DEVICE  open and use the specified DEVICE instead of stdin\n\
+      --help         display this help and exit\n\
+      --version      output version information and exit\n\
 \n\
 Optional - before SETTING indicates negation.  An * marks non-POSIX\n\
 settings.  The underlying system defines which settings are available.\n\
@@ -671,7 +680,7 @@ valid_options (char *opt, const char *valid_opts,
 {
   char ch;
 
-  if (*opt++ != '-')
+  if (*opt++ != '-' || *opt == 0)
     return 0;
 
   while ((ch = *opt))
@@ -709,6 +718,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
   parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
 		      AUTHORS, usage);
 
@@ -1434,13 +1445,20 @@ screen_columns (void)
      (but it works for ptys).
      It can also fail on any system when stdout isn't a tty.
      In case of any failure, just use the default.  */
-  if (get_win_size (STDOUT_FILENO, &win) == 0 && win.ws_col > 0)
+  if (get_win_size (STDOUT_FILENO, &win) == 0 && 0 < win.ws_col)
     return win.ws_col;
 #endif
-  /* FIXME: use xstrtol */
-  if (getenv ("COLUMNS"))
-    return atoi (getenv ("COLUMNS"));
-  return 80;
+  {
+    /* Use $COLUMNS if it's in [1..INT_MAX-1].  */
+    char *col_string = getenv ("COLUMNS");
+    long n_columns;
+    if (!(col_string != NULL
+	  && xstrtol (col_string, NULL, 0, &n_columns, "") == LONGINT_OK
+	  && 0 < n_columns
+	  && n_columns < INT_MAX))
+      n_columns = 80;
+    return n_columns;
+  }
 }
 
 static tcflag_t *
@@ -1801,7 +1819,7 @@ visible (unsigned int ch)
   char *bpout = buf;
 
   if (ch == _POSIX_VDISABLE)
-    return _("<undef>");
+    return "<undef>";
 
   if (ch >= 32)
     {
